@@ -4,11 +4,9 @@
 	supports several commands designed for use with the Linux SCSI
 	tape drive.
 
-	Maintained by Kai Mkisara (email Kai.Makisara@kolumbus.fi)
-	Copyright by Kai Mkisara, 1998 - 2008. The program may be distributed
-	according to the GNU Public License
-
-	Last Modified: Sun Apr 27 19:49:00 2008 by kai.makisara
+	Maintained by Iustin Pop (iustin@k1024.org).
+	Copyright by Kai Mäkisara, 1998 - 2008. The program may be distributed
+	according to the GNU Public License.
 */
 
 #include <stdio.h>
@@ -18,22 +16,25 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
+#include <sys/sysmacros.h>
 
 #include "mtio.h"
+#include "version.h"
 
 #ifndef DEFTAPE
 #define DEFTAPE "/dev/tape"     /* default tape device */
 #endif /* DEFTAPE */
 
-#define VERSION "1.1"
+typedef struct cmdef_tr cmdef_tr;
 
-typedef int (* cmdfunc)(/* int, struct cmdef_tr *, int, char ** */);
+typedef int (* cmdfunc)(int, struct cmdef_tr *, int, char **);
 
-typedef struct cmdef_tr {
+struct cmdef_tr {
     char *cmd_name;
     int cmd_code;
     cmdfunc cmd_function;
@@ -41,7 +42,7 @@ typedef struct cmdef_tr {
     unsigned char cmd_fdtype;
     unsigned char arg_cnt;
     int error_tests;
-} cmdef_tr;
+};
 
 #define NO_FD      0
 #define FD_RDONLY  1
@@ -60,6 +61,7 @@ typedef struct cmdef_tr {
 #define ET_WPROT     2
 
 static void usage(int);
+static void version() __attribute__ ((noreturn));
 static int do_standard(int, cmdef_tr *, int, char **);
 static int do_drvbuffer(int, cmdef_tr *, int, char **);
 static int do_options(int, cmdef_tr *, int, char **);
@@ -162,9 +164,9 @@ static cmdef_tr cmds[] = {
     ET_ONLINE },
     { "asf",		0,         do_asf, MTREW,  FD_RDONLY, ONE_ARG,
     ET_ONLINE },
-    { "stshowopt",	0,         do_show_options, 0,  FD_RDONLY, ONE_ARG,
+    { "stshowoptions",	0,         do_show_options, 0,  FD_RDONLY, ONE_ARG,
     0 },
-    { NULL, 0, 0, 0 }
+    { NULL, 0, 0, 0, NO_FD, NO_ARGS, 0 }
 };
 
 
@@ -225,8 +227,25 @@ static struct densities {
     {0x45, "QIC-3095-MC (TR-4)"},
     {0x46, "LTO-4"},
     {0x47, "DDS-5 or TR-5"},
+    {0x48, "SDLT220"},
+    {0x49, "SDLT320"},
+    {0x4a, "SDLT600, T10000A"},
+    {0x4b, "T10000B"},
+    {0x4c, "T10000C"},
+    {0x4d, "T10000D"},
     {0x51, "IBM 3592 J1A"},
     {0x52, "IBM 3592 E05"},
+    {0x53, "IBM 3592 E06"},
+    {0x54, "IBM 3592 E07"},
+    {0x55, "IBM 3592 E08"},
+    {0x58, "LTO-5"},
+    {0x5a, "LTO-6"},
+    {0x5c, "LTO-7"},
+    {0x71, "IBM 3592 J1A, encrypted"},
+    {0x72, "IBM 3592 E05, encrypted"},
+    {0x73, "IBM 3592 E06, encrypted"},
+    {0x74, "IBM 3592 E07, encrypted"},
+    {0x75, "IBM 3592 E08, encrypted"},
     {0x80, "DLT 15GB uncomp. or Ecrix"},
     {0x81, "DLT 15GB compressed"},
     {0x82, "DLT 20GB uncompressed"},
@@ -269,15 +288,18 @@ static struct booleans {
 #endif
     {"sili",	      MT_ST_SILI,	   "enable SILI for variable block mode"},
     {"cleaning",      MT_ST_SET_CLN,	   "set the cleaning bit location and mask"},
-    {NULL, 0}};
+    {NULL, 0, NULL}
+};
 
 static char *tape_name;   /* The tape name for messages */
 
 
+
 	int
 main(int argc, char **argv)
 {
-    int mtfd, cmd_code, i, argn, len, oflags;
+    int mtfd, i, argn, oflags;
+    unsigned int len;
     char *cmdstr;
     cmdef_tr *comp, *comp2;
 
@@ -298,14 +320,12 @@ main(int argc, char **argv)
 		exit(0);
 		break;
 	    case 'v':
-		printf("mt-st v. %s\n", VERSION);
-		exit(0);
+		version();
 		break;
 	    case '-':
 		if (*(argv[argn] + 1) == '-' &&
 		    *(argv[argn] + 2) == 'v') {
-		    printf("mt-st v. %s\n", VERSION);
-		    exit(0);
+		    version();
 		}
 		/* Fall through */
 	    default:
@@ -315,8 +335,17 @@ main(int argc, char **argv)
 	else
 	    break;
 
-    if (tape_name == NULL && (tape_name = getenv("TAPE")) == NULL)
+    if (tape_name == NULL && (tape_name = getenv("TAPE")) == NULL) {
+	struct stat stbuf;
+
 	tape_name = DEFTAPE;
+	if (!stat(tape_name, &stbuf) &&
+	    !S_ISCHR(stbuf.st_mode)) {
+	    fprintf(stderr, "The default '%s' is not a character device.\n\n", tape_name);
+	    usage(1);
+	    exit(1);
+	}
+    }
        
     if (argn >= argc ) {
 	usage(0);
@@ -348,7 +377,6 @@ main(int argc, char **argv)
 		comp->cmd_name);
 	exit(1);
     }
-    cmd_code = comp->cmd_code;
 
     if (comp->cmd_fdtype != NO_FD) {
 	oflags = comp->cmd_fdtype == FD_RDONLY ? O_RDONLY : O_RDWR;
@@ -383,6 +411,14 @@ main(int argc, char **argv)
 }
 
 
+	static void
+version()
+{
+	printf("mt-st v. %s\n", VERSION);
+	printf("default tape device: '%s'\n", DEFTAPE);
+	exit(0);
+}
+
 	static void
 usage(int explain)
 {
@@ -418,21 +454,33 @@ usage(int explain)
 	static int
 do_standard(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 {
+    int multiplier, max_count;
     struct mtop mt_com;
     char *endp;
 
     mt_com.mt_op = cmd->cmd_code;
     mt_com.mt_count = (argc > 0 ? strtol(*argv, &endp, 0) : 1);
     if (argc > 0 && endp != *argv) {
+	multiplier = 1;
 	if (*endp == 'k')
-	    mt_com.mt_count *= 1024;
+	    multiplier = 1024;
 	else if (*endp == 'M')
-	    mt_com.mt_count *= 1024 * 1024;
+	    multiplier = 1024 * 1024;
 	else if (*endp == 'G')
-	    mt_com.mt_count *= 1024 * 1024 * 1024;
+	    multiplier = 1024 * 1024 * 1024;
+	else if (*endp != 0) {
+	    fprintf(stderr, "mt: illegal count unit.\n");
+	    return 3;
+	}
+	max_count = INT_MAX / multiplier;
+	if (abs(mt_com.mt_count) > max_count) {
+	    fprintf(stderr, "mt: repeat count too large.\n");
+	    return 3;
+	}
+	mt_com.mt_count *= multiplier;
     }
     mt_com.mt_count |= cmd->cmd_count_bits;
-    if (mt_com.mt_count < 0) {
+    if (mt_com.mt_op != MTMKPART && mt_com.mt_count < 0) {
 	fprintf(stderr, "mt: negative repeat count\n");
 	return 1;
     }
@@ -474,7 +522,8 @@ do_drvbuffer(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 	static int
 do_options(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 {
-    int i, an, len;
+    int i, an;
+    unsigned int len;
     struct mtop mt_com;
 
     mt_com.mt_op = MTSETDRVBUFFER;
@@ -528,7 +577,8 @@ do_options(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 
 /* Tell where the tape is */
 	static int
-do_tell(int mtfd, cmdef_tr *cmd, int argc, char **argv)
+do_tell(int mtfd, cmdef_tr *cmd __attribute__((unused)),
+       int argc __attribute__((unused)), char **argv __attribute__((unused)))
 {
     struct mtpos mt_pos;
 
@@ -543,7 +593,7 @@ do_tell(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 
 /* Position the tape to a specific location within a specified partition */
 	static int
-do_partseek(int mtfd, cmdef_tr *cmd, int argc, char **argv)
+do_partseek(int mtfd, cmdef_tr *cmd __attribute__((unused)), int argc, char **argv)
 {
     struct mtop mt_com;
 
@@ -566,7 +616,7 @@ do_partseek(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 /* Position to start of file n. This might be implemented more intelligently
    some day. */
 	static int
-do_asf(int mtfd, cmdef_tr *cmd, int argc, char **argv)
+do_asf(int mtfd, cmdef_tr *cmd __attribute__((unused)), int argc, char **argv)
 {
     struct mtop mt_com;
 
@@ -591,10 +641,13 @@ do_asf(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 /*** Decipher the status ***/
 
 	static int
-do_status(int mtfd, cmdef_tr *cmd, int argc, char **argv)
+do_status(int mtfd, cmdef_tr *cmd __attribute__((unused)),
+       int argc __attribute__((unused)),
+       char **argv __attribute__((unused)))
 {
     struct mtget status;
-    int dens, i;
+    int dens;
+    unsigned int i;
     char *type, *density;
 
     if (ioctl(mtfd, MTIOCGET, (char *)&status) < 0) {
@@ -696,7 +749,10 @@ static const char *st_formats[] = {
         "m", "v", "p", "x", "a", "y", "q", "z"}; 
 
 /* Show the options if visible in sysfs */
-static int do_show_options(int mtfd, cmdef_tr *cmd, int argc, char **argv)
+static int do_show_options(int mtfd,
+    cmdef_tr *cmd __attribute__((unused)),
+    int argc __attribute__((unused)),
+    char **argv __attribute__((unused)))
 {
     int i, fd, options, tapeminor, tapeno, tapemode;
     struct stat stat;
@@ -708,9 +764,6 @@ static int do_show_options(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 	return 2;
     }
     sscanf(uts.release, "%d.%d.%d", &i, &tapeno, &tapemode);
-    if (i < 2 || tapeno < 6 || tapemode < 26)
-	printf("Your kernel (%d.%d.%d) may be too old for this command.\n",
-	       i, tapeno, tapemode);
 
     if (fstat(mtfd, &stat) < 0) {
 	perror(tape_name);
@@ -751,9 +804,12 @@ static int do_show_options(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 
 /* Print a list of possible density codes */
 	static int
-print_densities(int fd, cmdef_tr *cmd, int argc, char **argv)
+print_densities(int fd __attribute__((unused)),
+       cmdef_tr *cmd __attribute__((unused)),
+       int argc __attribute__((unused)),
+       char **argv __attribute__((unused)))
 {
-    int i, offset;
+    unsigned int i, offset;
 
     printf("Some SCSI tape density codes:\ncode   explanation                   code   explanation\n");
     offset = (NBR_DENSITIES + 1) / 2;
