@@ -1,11 +1,11 @@
 /* This program initializes Linux SCSI tape drives using the
    inquiry data from the devices and a text database.
 
-   Copyright 1996-2008 by Kai Mkisara (email Kai.Makisara@kolumbus.fi)
+   Maintained by Iustin Pop (iustin@k1024.org).
+   Copyright 1996-2008 by Kai Mäkisara (Kai.Makisara@kolumbus.fi).
    Distribution of this program is allowed according to the
    GNU Public Licence.
 
-   Last modified: Sun Apr 27 14:24:16 2008 by kai.makisara
 */
 
 #include <stdio.h>
@@ -16,6 +16,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <dirent.h>
+#include <limits.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/sysmacros.h>
@@ -23,6 +24,7 @@
 #include <scsi/sg.h>
 
 #include "mtio.h"
+#include "version.h"
 
 #ifndef FALSE
 #define TRUE 1
@@ -30,8 +32,6 @@
 #endif
 #define SKIP_WHITE(p) for ( ; *p == ' ' || *p == '\t'; p++)
 
-#define VERSION "1.1"
-
 typedef struct _modepar_tr {
     int defined;
     int blocksize;
@@ -148,7 +148,7 @@ find_string(char *s, char *target, char *buf, int buflen)
 		else
 		    for (cp2=cp+1; isalnum(*cp2) || *cp2 == '-'; cp2++)
 			;
-		if (cp2 == '\0')
+		if (*cp2 == '\0')
 		    return NULL;
 		have_arg = TRUE;
 		argp = cp;
@@ -164,7 +164,7 @@ find_string(char *s, char *target, char *buf, int buflen)
 		if (buf == NULL)
 		    buf = strdup(argp);
 		else {
-		    if (strlen(argp) < buflen)
+		    if (strlen(argp) < (unsigned int)buflen)
 			strcpy(buf, argp);
 		    else {
 			strncpy(buf, argp, buflen);
@@ -209,9 +209,9 @@ num_arg(char *t)
 
 
 	static int
-next_block(FILE *dbf, char *buf, int buflen, int limiter)
+next_block(FILE *dbf, char *buf, size_t buflen, int limiter)
 {
-    int len;
+    size_t len;
     char *cp, *bp;
     static char lbuf[LINEMAX];
 
@@ -231,7 +231,7 @@ next_block(FILE *dbf, char *buf, int buflen, int limiter)
 	    memmove(lbuf, cp, strlen(cp) + 1);
 	    return TRUE;
 	}
-	if (len + strlen(lbuf) >= DEFMAX) {
+	if (len + strlen(lbuf) >= buflen) {
 	    fprintf(stderr, "Too long definition: '%s'\n", buf);
 	    return FALSE;
 	}
@@ -255,7 +255,7 @@ find_pars(FILE *dbf, char *company, char *product, char *rev, devdef_tr *defs,
 	  int parse_only)
 {
     int i, mode, modes_defined, errors;
-    char line[LINEMAX], defstr[DEFMAX], comdef[DEFMAX];
+    char line[LINEMAX], defstr[DEFMAX], comdef[DEFMAX], modebuf[DEFMAX];
     char tmpcomp[LINEMAX], tmpprod[LINEMAX], tmprev[LINEMAX], *cp, c, *t;
     char *nextdef, *curdef, *comptr;
     static int call_nbr = 0;
@@ -349,70 +349,70 @@ find_pars(FILE *dbf, char *company, char *product, char *rev, devdef_tr *defs,
 		continue;
 	    }
 
-	    strcpy(defstr, comptr);
-	    strcat(defstr, cp);
+	    strcpy(modebuf, comptr);
+	    strcat(modebuf, cp);
 	    *nextdef = c;
 
 	    if (verbose > 1)
-		fprintf(stderr, "Mode %d definition: %s\n", mode + 1, defstr);
+		fprintf(stderr, "Mode %d definition: %s\n", mode + 1, modebuf);
 
-	    if ((t = find_string(defstr, "disab", line, LINEMAX)) != NULL &&
+	    if ((t = find_string(modebuf, "disab", line, LINEMAX)) != NULL &&
 		strtol(t, NULL, 0) != 0) {
 		defs->modedefs[mode].defined = FALSE;
 		continue;
 	    }
 
-	    if ((t = find_string(defstr, "drive-", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "drive-", line, LINEMAX)) != NULL)
 		defs->drive_buffering = num_arg(t);
-	    if ((t = find_string(defstr, "timeout", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "timeout", line, LINEMAX)) != NULL)
 		defs->timeout = num_arg(t);
-	    if ((t = find_string(defstr, "long-time", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "long-time", line, LINEMAX)) != NULL)
 		defs->long_timeout = num_arg(t);
-	    if ((t = find_string(defstr, "clean", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "clean", line, LINEMAX)) != NULL)
 		defs->cleaning = num_arg(t);
-	    if ((t = find_string(defstr, "no-w", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "no-w", line, LINEMAX)) != NULL)
 		defs->nowait = num_arg(t);
-	    if ((t = find_string(defstr, "sili", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "sili", line, LINEMAX)) != NULL)
 		defs->sili = num_arg(t);
 
 	    defs->modedefs[mode].defined = TRUE;
-	    if ((t = find_string(defstr, "block", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "block", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].blocksize = num_arg(t);
-	    if ((t = find_string(defstr, "dens", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "dens", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].density = num_arg(t);
-	    if ((t = find_string(defstr, "buff", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "buff", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].buffer_writes = num_arg(t);
-	    if ((t = find_string(defstr, "async", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "async", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].async_writes = num_arg(t);
-	    if ((t = find_string(defstr, "read", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "read", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].read_ahead = num_arg(t);
-	    if ((t = find_string(defstr, "two", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "two", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].two_fm = num_arg(t);
-	    if ((t = find_string(defstr, "comp", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "comp", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].compression = num_arg(t);
-	    if ((t = find_string(defstr, "auto", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "auto", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].auto_lock = num_arg(t);
-	    if ((t = find_string(defstr, "fast", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "fast", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].fast_eod = num_arg(t);
-	    if ((t = find_string(defstr, "can-b", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "can-b", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].can_bsr = num_arg(t);
-	    if ((t = find_string(defstr, "noblk", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "noblk", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].no_blklimits = num_arg(t);
-	    if ((t = find_string(defstr, "can-p", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "can-p", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].can_partitions = num_arg(t);
-	    if ((t = find_string(defstr, "scsi2", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "scsi2", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].scsi2logical = num_arg(t);
-	    if ((t = find_string(defstr, "sysv", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "sysv", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].sysv = num_arg(t);
-	    if ((t = find_string(defstr, "defs-for-w", line, LINEMAX)) != NULL)
+	    if ((t = find_string(modebuf, "defs-for-w", line, LINEMAX)) != NULL)
 		defs->modedefs[mode].defs_for_writes = num_arg(t);
 
-	    for (t=defstr; *t == ' ' || *t == '\t'; t++)
+	    for (t=modebuf; *t == ' ' || *t == '\t'; t++)
 		;
 	    if (*t != '\0' && call_nbr <= 1) {
 		fprintf(stderr,
 			"Warning: errors in definition for ('%s', '%s', '%s'):\n%s\n",
-			tmpcomp, tmpprod, tmprev, defstr);
+			tmpcomp, tmpprod, tmprev, modebuf);
 		errors++;
 	    }
 	}
@@ -451,9 +451,6 @@ find_pars(FILE *dbf, char *company, char *product, char *rev, devdef_tr *defs,
 
 static int sg_io_errcheck(struct sg_io_hdr *hdp)
 {
-    int status;
-
-    status = hdp->status & 0x7e;
     if ((hdp->status & 0x7e) == 0 || hdp->host_status == 0 ||
 	hdp->driver_status == 0)
 	return 0;
@@ -889,7 +886,7 @@ define_tape(int tapeno, FILE *dbf, devdef_tr *defptr, int print_non_found)
 usage(int retval)
 {
     fprintf(stderr,
-	    "Usage: stinit [-h] [-v] [-f dbname] [-p] [drivename_or_number ...]\n");
+	    "Usage: stinit [-h] [-v] [--version] [-f dbname] [-p] [-r] [drivename_or_number ...]\n");
     exit(retval);
 }
 
